var game = {
    tmr: null,
    speed: 10,
    balls: {},
    canvas: null,
    ctx: null,
    shift: false,
    gamepads: {},
    joysticks: []
};
var viewer = {
    buttons: [],
    axes: [],
    mid: [],
    joysticks: [],
    axRadius: 20,
    axisCanvas: null,
    ctxAxis: null
};
var Joystick = function(lra, uda, lrn, udn) {
    this.lra = lra; // left right axes
    this.uda = uda; // up down axes
    this.lrn = lrn; // left right neutral value
    this.udn = udn; // up down neutral value
};
var Ball = function(colour, x, y){
    this.x = x;
    this.y = y;
    this.colour = colour;
    this.moveX = 0;
    this.moveY = 0;
    this.radius = 12;
    this.maxMove = 4;
};
Ball.prototype.animate = function(){
    this.x += this.moveX;
    this.y += this.moveY;
    if(this.x > game.canvas.width - this.radius){
        this.x = game.canvas.width - this.radius;
    } else if (this.x - this.radius < 0){
        this.x = this.radius;
    }
    if(this.y > game.canvas.height - this.radius){
        this.y = game.canvas.height - this.radius;
    } else if(this.y - this.radius < 0){
        this.y = this.radius;
    }
    game.ctx.beginPath();
    game.ctx.arc(this.x, this.y, this.radius, 0, Math.PI * 2);
    game.ctx.fillStyle = this.colour;
    game.ctx.fill();
    game.ctx.closePath();
};
function keyPressed(e){
    let ball = game.shift ? 1 : 0;
    switch(e.which){
        case 16: // shift
            game.shift = true;
            game.balls[0].moveX = game.balls[0].moveY = 0;
            break;
        case 37: //left
            game.balls[ball].moveX = -game.balls[ball].maxMove;
            break;
        case 39: // right
            game.balls[ball].moveX = game.balls[ball].maxMove;
            break;
        case 38: // up
            game.balls[ball].moveY = -game.balls[ball].maxMove;
            break;
        case 40: // down
            game.balls[ball].moveY = game.balls[ball].maxMove;
            break;
        case 87: // w and up
            game.balls[1].moveY = -game.balls[1].maxMove;
            break;
        case 90: // z and down
            game.balls[1].moveY = game.balls[1].maxMove;
            break;
        case 65: // a and left
            game.balls[1].moveX = -game.balls[1].maxMove;
            break;
        case 83: // s and right
            game.balls[1].moveX = game.balls[1].maxMove;
            break;
    }
}
function keyReleased(e){
    let ball = game.shift ? 1 : 0;
    switch(e.which){
        case 16:
            game.shift = false;
            game.balls[1].moveX = game.balls[1].moveY = 0;
            break;
        case 37:
        case 39:
            game.balls[ball].moveX = 0;
            break;
        case 38:
        case 40:
            game.balls[ball].moveY = 0;
            break;
        case 87:
        case 90:
            game.balls[1].moveY = 0;
            break;
        case 65:
        case 83:
            game.balls[1].moveX = 0;
            break;
    }
}
function updateViewer() {
    for(let g in game.gamepads){
        let gpad = game.gamepads[g];
        updateButtons(gpad);
        updateAxesValues(gpad);
        drawJoysticks(gpad);
        break;
    }
}
function drawJoysticks(gpad){
    viewer.ctxAxis.clearRect(0, 0, viewer.axisCanvas.width, viewer.axisCanvas.height);
    for(let a = 0; a < 2; a++){
        let lr = game.joysticks[a].lra;
        let ud = game.joysticks[a].uda;
        drawAxes(viewer.joysticks[a], 
            (gpad.axes[lr] === game.joysticks[a].lrn) ? 0 : gpad.axes[lr], 
            (gpad.axes[ud] === game.joysticks[a].udn) ? 0 : gpad.axes[ud]);
    }
}
function updateAxesValues(gpad){
    for(let a = 0; a < gpad.axes.length; a++) {
        viewer.axes[a].html(gpad.axes[a].toFixed(4));
        if(gpad.axes[a] - viewer.mid[a] < 0){
            viewer.axes[a].removeClass("lbl50").addClass("lblLeft");
        } else {
            if(gpad.axes[a] - viewer.mid[a] > 0){
                viewer.axes[a].removeClass("lbl50").addClass("lblRight");
            } else {
                viewer.axes[a].removeClass("lblLeft lblRight").addClass("lbl50");
            }
        }
    }
}
function updateButtons(gpad){
    for(let b = 0; b < gpad.buttons.length; b++){
        let val = gpad.buttons[b];
        let pressed = (val === 1.0);
        if (typeof(val) === "object") {
            pressed = val.pressed;
          }
        if(pressed){
            viewer.buttons[b].removeClass("btnNorm").addClass("btnPress");
        } else {
            viewer.buttons[b].removeClass("btnPress").addClass("btnNorm");
        }
    }
}
function drawAxes(pos, valX, valY){
    let x = pos.x + Math.round(valX * viewer.axRadius + 0.49 * Math.sign(valX));
    let y = pos.y + Math.round(valY * viewer.axRadius + 0.49 * Math.sign(valY));
    viewer.ctxAxis.beginPath();
    viewer.ctxAxis.arc(pos.x, pos.y, viewer.axRadius, 0, Math.PI * 2);
    viewer.ctxAxis.stroke();
    viewer.ctxAxis.closePath();
    if(valX !== 0 || valY !== 0 ){
        viewer.ctxAxis.beginPath();
        viewer.ctxAxis.moveTo(pos.x, pos.y);
        viewer.ctxAxis.lineTo(x, y);
        viewer.ctxAxis.stroke();
        viewer.ctxAxis.closePath();
    }
}
function redraw() {
    game.ctx.clearRect(0, 0, game.canvas.width, game.canvas.height);
    for(let b in game.balls){
        let ball = game.balls[b];
        ball.animate();
    }
    updateViewer();
}
function resizeCanvas() {
    game.canvas.width = window.innerWidth;
    game.canvas.height = window.innerHeight - parseInt($("#dvButtonView").css("height"));
}
function scanGamepads() {
    let gpads = navigator.getGamepads ? navigator.getGamepads() : (navigator.webkitGetGamepads ? navigator.webkitGetGamepads() : []);
    for (let i = 0; i < gpads.length; i++) {
      if (gpads[i]) {
        if (!(gpads[i].index in game.gamepads)) {
          addGamepad(gpads[i]);
        } else {
          game.gamepads[gpads[i].index] = gpads[i];
        }
      }
    }
}
function getGamepadInput() {
    let gamepadFound = false;
    scanGamepads();
    for(let g in game.gamepads){
        let gpad = game.gamepads[g];
        gamepadFound = true;
        for(let a = 0; a < 2; a++){
            let lr = game.joysticks[a].lra;
            let ud = game.joysticks[a].uda;
            let ball = game.balls[a];
            ball.moveX = (gpad.axes[lr] === game.joysticks[a].lrn) ? 0 : gpad.axes[lr] * ball.maxMove;
            ball.moveY = (gpad.axes[ud] === game.joysticks[a].udn) ? 0 : gpad.axes[ud] * ball.maxMove;
        }
    }
    if(gamepadFound){
        window.requestAnimationFrame(getGamepadInput);
    }    
}
function addJoysticks(gamepad) {
    game.joysticks.push(new Joystick(0, 1, gamepad.axes[0], gamepad.axes[1]));
    game.joysticks.push(new Joystick(2, 5, gamepad.axes[2], gamepad.axes[5]));
}
function addButtonView(gamepad){
    let bv = $("#dvButtonView");
    for(let b = 0; b < gamepad.buttons.length; b++){
        let btn = $("<button>").html('' +(b+1)).addClass("btnNorm").css("left", (b * 70) + 'px');
        viewer.buttons.push(btn);
        bv.append(btn);
    }
    let diam = viewer.axRadius * 2;
    viewer.axisCanvas.width = 2 * (diam + 20);
    viewer.axisCanvas.height = diam + 2;
    for(let a = 0; a < 2; a++){
        let ax = {x: a * diam + a * 20 + diam, y: viewer.axRadius + 1};
        viewer.joysticks.push(ax);
    }
    for (let a = 0; a < gamepad.axes.length; a++){
        let lbl = $("<label>").html("0.000").addClass("lbl50").css("left", (a * 70) + 'px');
        viewer.axes.push(lbl);
        viewer.mid.push(gamepad.axes[a]); // store neutral position value
        bv.append(lbl);
    }
}
function addGamepad(gamepad){
    game.gamepads[gamepad.index] = gamepad;
    if(Object.keys(game.gamepads).length == 1){
        addJoysticks(gamepad); // add joysticks for first attached
        addButtonView(gamepad); // add button view for first attached
    }
    window.requestAnimationFrame(getGamepadInput);
}
function deleteGamepad(gamepad){
    delete game.gamepads[gamepad.index];
}

$.when( $.ready ).then(function() {
    game.balls[0] = new Ball("red", 50, 50);
    game.balls[1] = new Ball("blue", 150, 50);
    game.canvas = document.getElementById("playArea");
    game.ctx = game.canvas.getContext("2d");
    $("body").keydown(function(e){keyPressed(e);});
    $("body").keyup(function(e){keyReleased(e);});
    viewer.axisCanvas = document.getElementById("cvAxis");
    viewer.ctxAxis = viewer.axisCanvas.getContext("2d");
    window.addEventListener("gamepadconnected", function(e) {
        addGamepad(e.gamepad);
    });
    window.addEventListener("gamepaddisconnected", function(e) {
       deleteGamepad(e.gamepad);
    });
    resizeCanvas();
    game.tmr = setInterval(redraw, game.speed);   
});
